﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(CharacterController))]
public class PlayerController : MonoBehaviour
{
    public float distance = 10.0f;


    float velocityY = 0.0f;
    const float gravity = 9.8f;

    public HookBehaviour hook;

    private Rigidbody rb;
	private CapsuleCollider col;
	private DistanceJoint3D joint; 
    private bool isWired;


    
	[SerializeField]
	private Transform groundChecker;
	[SerializeField]
	private LayerMask groundMask;
	private bool IsGrounded
	{
		get {
			return Physics.OverlapSphere(groundChecker.position, 0.01f, groundMask).Length > 0;
		}
	}

    [SerializeField]
    private float bulletMaxDistance;
    [SerializeField]
    private ParticleSystem bulletEffect;
    [SerializeField]
	private LayerMask enemyMask;
    [SerializeField]
    private LayerMask bulletBlockMask;

#if UNITY_EDITOR
    private void OnDrawGizmos()
    {
		Gizmos.color = Color.red;
        Gizmos.DrawLine(transform.position, Camera.main.transform.forward.normalized * 200);

		if (!IsGrounded)
			Gizmos.color = Color.green;
		Gizmos.DrawSphere(groundChecker.position, 0.1f);
	}
#endif

    private void Start()
    {
        Cursor.lockState = CursorLockMode.Locked;
        joint = GetComponent<DistanceJoint3D>();
        rb = GetComponent<Rigidbody>();
		col = GetComponent<CapsuleCollider>();

		//col.enabled = !IsGrounded;
		//cc.enabled = IsGrounded;
    }
    private void Update()
    {
        float vertical = Input.GetAxis("Vertical");
        float horizontal = Input.GetAxis("Horizontal");

     
        RaycastHit[] enemyHits = Physics.SphereCastAll(Camera.main.ScreenPointToRay(Input.mousePosition), 3, bulletMaxDistance, enemyMask);

        List<Enemy> hitEnemies = new List<Enemy>();
        List<Vector3> hitEnemyPositions = new List<Vector3>();

        foreach (var enemyHit in enemyHits)
        {
            Ray ray = new Ray(Camera.main.transform.position, (enemyHit.point - Camera.main.transform.position).normalized);
            if (!Physics.Raycast(ray, Vector3.Distance(Camera.main.transform.position, enemyHit.point), bulletBlockMask))
            {
                hitEnemies.Add(enemyHit.collider.GetComponent<Enemy>());
                hitEnemyPositions.Add(enemyHit.transform.position);
            }
        }

		IngameUIManager.inst.UpdateTargetLockedUIs(hitEnemyPositions.ToArray());


		if (Input.GetMouseButton(0))
		{
            bulletEffect.transform.rotation = Camera.main.transform.rotation;
            bulletEffect.Play();
			foreach (var enemy in hitEnemies)
			{
				enemy?.GetDamaged(1);
			}
		}

        if (Input.GetMouseButtonDown(1))
        {
            if (!hook.gameObject.activeSelf)
            {
                FireHook();
            }
            else
                ReturnHook();
        }
		
		if (IsGrounded)
		{
			rb.velocity = (horizontal * transform.right + vertical * transform.forward).normalized * 10.0f;
			if (Input.GetButtonDown("Jump"))
				rb.velocity += new Vector3(0, 5 -rb.velocity.y, 0);
		}
        else
        {
			rb.AddForce(horizontal * transform.right + vertical * transform.forward * 10.0f);
        }
	}

    private void LateUpdate()
    {
        IngameUIManager.inst.UpdateHeightText(transform.position.y);
        if (GameManager.inst.MaxHeight < transform.position.y)
        {
            IngameUIManager.inst.UpdateMaxHeightText(transform.position.y);
            GameManager.inst.MaxHeight = transform.position.y;
        }
    }

    private void FireHook()
    {
        joint.enabled = false;
        hook.gameObject.SetActive(true);
        hook.transform.position = transform.position;
        hook.EnableHook(Camera.main.transform.forward.normalized * 200);
    }

    public void ReturnHook()
    {
        joint.enabled = false;
        hook.DisableHook();
        isWired = false;
        rb.useGravity = true;
    }

    public void ActiveWire()
    {
        joint.enabled = true;
        joint.distance = Vector3.Distance(transform.position, hook.transform.position);
        isWired = true;
        rb.useGravity = false;
    }
}
